library(here)
library(tidyverse)
library(phyloseq)
library(DESeq2)
library(qiime2R)

#Calculate relative abundance
calculate_relative_abundance <- function(x) x / sum(x)

#Calculate geometric mean 
calculate_gm_mean <- function(x, na.rm = TRUE) {
  exp(sum(log(x[x > 0]), na.rm = na.rm) / length(x))
}

#Import data
(capt_physeq<-qza_to_phyloseq(
  features=here::here("captivity_filtered_tables","capt-filtered-table.qza"),
  tree=here::here("rooted-tree.qza"),
  taxonomy=here::here("taxonomy.qza"),
  metadata=here::here("zoo_metadata_HL.txt")))
(wild_physeq<-qza_to_phyloseq(
  features=here::here("captivity_filtered_tables","wild-filtered-table.qza"),
  tree=here::here("rooted-tree.qza"),
  taxonomy=here::here("taxonomy.qza"),
  metadata=here::here("zoo_metadata_HL.txt")))

capt_counts <- taxa_sums(capt_physeq)
relative_abundance_capt <- calculate_relative_abundance(capt_counts)
abundant_capt <- relative_abundance_capt > 0.0005
abundant_capt_taxa <- prune_taxa(abundant_capt, capt_physeq)

wild_counts <- taxa_sums(wild_physeq)
relative_abundance_wild <- calculate_relative_abundance(wild_counts)
abundant_wild <- relative_abundance_wild > 0.0005
abundant_wild_taxa <- prune_taxa(abundant_wild, wild_physeq)

#Remove "uncultured" taxa
subset_abundant_capt_taxa <- subset_taxa(abundant_capt_taxa, Genus != "uncultured")

abundant_capt_genera <- tax_glom(subset_abundant_capt_taxa, taxrank = "Genus")
deseq_capt <- phyloseq_to_deseq2(abundant_capt_genera, ~ ConservationStatusHMC)
abundant_wild_genera <- tax_glom(abundant_wild_taxa, taxrank = "Genus")
deseq_wild <- phyloseq_to_deseq2(abundant_wild_genera, ~ ConservationStatusHMC)

#Setting reference = LCNT
deseq_capt$ConservationStatusHMC <- relevel(deseq_capt$ConservationStatusHMC, ref = "LCNT")
deseq_wild$ConservationStatusHMC <- relevel(deseq_wild$ConservationStatusHMC, ref = "LCNT")

geo_means <- apply(counts(deseq_capt), 1, calculate_gm_mean)
deseq_capt <- estimateSizeFactors(deseq_capt, geoMeans = geo_means)
deseq_capt <- DESeq(deseq_capt, fitType = "local")

geo_means <- apply(counts(deseq_wild), 1, calculate_gm_mean)
deseq_wild <- estimateSizeFactors(deseq_wild, geoMeans = geo_means)
deseq_wild <- DESeq(deseq_wild, fitType = "local")

capt_diff_abund <- results(deseq_capt, name = "ConservationStatusHMC_VECE_vs_LCNT")
wild_diff_abund <- results(deseq_wild, name = "ConservationStatusHMC_VECE_vs_LCNT")

#Set significance level = 0.05
alpha <- 0.05
(significant_capt <- capt_diff_abund %>% as.data.frame() %>% filter(padj < alpha))
(significant_wild <- wild_diff_abund %>% as.data.frame() %>% filter(padj < alpha))

#Order abundance change in ascending order
genera_df_capt <- as.data.frame(tax_table(abundant_capt_genera))
(significant_capt <- significant_capt %>% 
    merge(genera_df_capt, by = "row.names") %>% 
    arrange(desc(log2FoldChange)))

genera_df_wild <- as.data.frame(tax_table(abundant_wild_genera))
(significant_wild <- significant_wild %>% 
    merge(genera_df_wild, by = "row.names") %>% 
    arrange(desc(log2FoldChange)))

significant_capt <- mutate(significant_capt,
                           Genus = factor(Genus, levels = Genus))
significant_wild <- mutate(significant_wild,
                           Genus = factor(Genus, levels = Genus))

#Sets colors for plots
capt_colors <- RColorBrewer::brewer.pal(8, "Set1")[1:2] %>% rev()
wild_colors <- RColorBrewer::brewer.pal(8, "Dark2")[1:2] %>% rev()

#Create plots with different colors based on group with more abundance
ggplot(significant_capt, aes(x = log2FoldChange, y = Genus,
                             fill = factor(ifelse(log2FoldChange<0,"low-risk","high-risk")))) +
  geom_bar(stat = "identity") +
  labs(x = expression(log[2]~fold~change),
       y = "Genus", fill = "More abundant in") +
  scale_fill_manual(values=capt_colors,
                    guide = guide_legend(reverse = TRUE)) +
  theme_bw(base_size = 20) +
  theme(panel.grid.major = element_blank(), panel.grid.minor = element_blank()) +
  scale_x_continuous(breaks = seq(-30, 30, by = 5))

ggplot(significant_wild, aes(x = log2FoldChange, y = Genus,
                             fill = factor(ifelse(log2FoldChange<0,"low-risk","high-risk")))) +
  geom_bar(stat = "identity") +
  labs(x = expression(log[2]~fold~change),
       y = "Genus", fill = "More abundant in") +
  scale_fill_manual(values=wild_colors,
                    guide = guide_legend(reverse = TRUE)) +
  theme_bw(base_size = 20) +
  theme(panel.grid.major = element_blank(), panel.grid.minor = element_blank()) +
  scale_x_continuous(breaks = seq(-30, 30, by = 5))
