library(here)
library(tidyverse)
library(microeco)
library(file2meco)
library(GUniFrac)

#Fix the randomization for reproducible results
set.seed(711)
theme_set(theme_bw(base_size=18))
calculate_relative_abundance <- function(x) x/sum(x)

#Import data
(meco_data <- qiime2meco(
  ASV_data=here::here("table.qza"),
  phylo_tree=here::here("rooted-tree.qza"),
  taxonomy_data=here::here("taxonomy.qza"),
  sample_data = here::here("zoo_metadata_microeco.txt")))
(meco_ungrouped_data <- qiime2meco(
  ASV_data=here::here("table.qza"),
  phylo_tree=here::here("rooted-tree.qza"),
  taxonomy_data=here::here("taxonomy.qza"),
  sample_data = here::here("zoo_metadata_microeco_ungrouped.txt")))

#Create dummy copies of the dataset
(capt_dataset <- clone(meco_data)) #for captivity status analyses
(wild_dataset <- clone(meco_data)) #for captivity status analyses
(cons_dataset <- clone(meco_ungrouped_data)) #for conservation status analyses


## Define minimum sequencing depth and check if TRUE:
#Output should be TRUE, thus no filtering by reads was done at this point
min_seq_depth <- 1000
all(capt_dataset$sample_sums() > min_seq_depth)


## Data preparation:
#Filter by metadata
capt_dataset$sample_table <- capt_dataset$sample_table %>%
  filter(captive_wild == "captive")
wild_dataset$sample_table <- wild_dataset$sample_table %>%
  filter(captive_wild == "wild")
capt_dataset$tidy_dataset()
wild_dataset$tidy_dataset()

#Rarefy the dataset at same depth as for QIIME2 analysis
capt_dataset$rarefy_samples(sample.size = 80508)
wild_dataset$rarefy_samples(sample.size = 80508)
cons_dataset$rarefy_samples(sample.size = 83570)

#Calculate relative abundance and remove low-abundant features
capt_dataset$cal_abund(rel=T)
wild_dataset$cal_abund(rel=T)
cons_dataset$cal_abund(rel=T)
total_capt_counts <- capt_dataset$taxa_sums()
total_wild_counts <- wild_dataset$taxa_sums()
total_cons_counts <- cons_dataset$taxa_sums()
relative_abundance <- calculate_relative_abundance(total_capt_counts)
abundant <- relative_abundance > 0.0005 
capt_dataset$otu_table <- capt_dataset$otu_table [abundant,]
capt_dataset$tidy_dataset()
wild_dataset$otu_table <- wild_dataset$otu_table [abundant,]
wild_dataset$tidy_dataset()
cons_dataset$otu_table <- cons_dataset$otu_table [abundant,]
cons_dataset$tidy_dataset()

#Create copies of the dataset for alpha and beta diversity analyses
(capt_alpha <- clone(capt_dataset))
(wild_alpha <- clone(wild_dataset))
(capt_beta <- clone(capt_dataset))
(wild_beta <- clone(wild_dataset))
(cons_beta <- clone(cons_dataset))


## Alpha diversity barplots according to captivity status:
#Prepare dataset for alpha diversity analysis
capt_alpha$cal_alphadiv(PD=T)
wild_alpha$cal_alphadiv(PD=T)
t_capt_alpha <- trans_alpha$new(dataset = capt_alpha, group = "ConservationStatusHMC")
t_wild_alpha <- trans_alpha$new(dataset = wild_alpha, group = "ConservationStatusHMC")

#Plot Shannon alpha diversity
t_capt_alpha$cal_diff(method = "KW", measures = "Shannon")
t_capt_alpha$plot_alpha(pair_compare = TRUE, measure = "Shannon",
                        color_values = capt_colors,
                        shape = "ConservationStatusHMC") + theme_bw() +
  theme(panel.grid.major = element_blank(), panel.grid.minor = element_blank(),
        legend.position = "none", text = element_text(size = 16)) +
  scale_y_continuous(labels = scales::number_format(accuracy = 0.1))
t_wild_alpha$cal_diff(method = "KW", measures = "Shannon")
t_wild_alpha$plot_alpha(pair_compare = TRUE, measure = "Shannon",
                        shape = "ConservationStatusHMC") + theme_bw() +
  theme(panel.grid.major = element_blank(), panel.grid.minor = element_blank(),
        legend.position = "none", text = element_text(size = 16))


## Venn diagram of shared taxa between groups:
capt_dataset_genus <- capt_dataset$merge_taxa(taxa = "Genus")
wild_dataset_genus <- wild_dataset$merge_taxa(taxa = "Genus")

#Merge according to conservation status
capt_merge_dataset <-
  capt_dataset_genus$merge_samples(use_group = "ConservationStatusHMC")
t_capt <- trans_venn$new(capt_merge_dataset, ratio = "numratio")
wild_merge_dataset <-
  wild_dataset_genus$merge_samples(use_group = "ConservationStatusHMC")
t_wild <- trans_venn$new(wild_merge_dataset, ratio = "numratio")

#View Venn diagrams
t_capt$plot_venn(color_circle = RColorBrewer::brewer.pal(8, "Set1"))
t_wild$plot_venn(color_circle = RColorBrewer::brewer.pal(8, "Dark2"))


## Data preparation for beta diversity analyses:
#Make separate copies for Jaccard and Bray-Curtis plots
capt_beta$cal_betadiv(unifrac = T)
wild_beta$cal_betadiv(unifrac = T)
cons_beta$cal_betadiv(unifrac = T)
t_capt_beta_jc <- trans_beta$new(dataset = capt_beta,
                                 group = "ConservationStatusHMC", measure = "jaccard")
t_wild_beta_jc <- trans_beta$new(dataset = wild_beta,
                                 group = "ConservationStatusHMC", measure = "jaccard")
t_cons_beta_jc <- trans_beta$new(dataset = cons_beta,
                                 group = "ConservationStatusHMC", measure = "jaccard")
t_capt_beta_bc <- trans_beta$new(dataset = capt_beta,
                                 group = "ConservationStatusHMC", measure = "bray")
t_wild_beta_bc <- trans_beta$new(dataset = wild_beta,
                                 group = "ConservationStatusHMC", measure = "bray")
t_cons_beta_bc <- trans_beta$new(dataset = cons_beta,
                                 group = "ConservationStatusHMC", measure = "bray")

#Make separate copies for unweighted and weighted UniFrac plots
t_capt_beta_uu <- trans_beta$new(dataset = capt_beta,
                                 group = "ConservationStatusHMC",
                                 measure = "unwei_unifrac")
t_wild_beta_uu <- trans_beta$new(dataset = wild_beta,
                                 group = "ConservationStatusHMC",
                                 measure = "unwei_unifrac")
t_cons_beta_uu <- trans_beta$new(dataset = cons_beta,
                                 group = "ConservationStatusHMC",
                                 measure = "unwei_unifrac")
t_capt_beta_wu <- trans_beta$new(dataset = capt_beta,
                                 group = "ConservationStatusHMC",
                                 measure = "wei_unifrac")
t_wild_beta_wu <- trans_beta$new(dataset = wild_beta,
                                 group = "ConservationStatusHMC",
                                 measure = "wei_unifrac")
t_cons_beta_wu <- trans_beta$new(dataset = cons_beta,
                                 group = "ConservationStatusHMC",
                                 measure = "wei_unifrac")

#Sets colors for plots
capt_colors <- RColorBrewer::brewer.pal(8, "Set1")[1:2]
wild_colors <- RColorBrewer::brewer.pal(8, "Dark2")[1:2]


## Beta diversity plots according to captivity status:
#PCoA plots for captive animals
t_capt_beta_jc$cal_ordination(ordination = "PCoA")
t_capt_beta_jc$plot_ordination(color_values = capt_colors,
                               plot_color = "ConservationStatusHMC", 
                               plot_shape = "ConservationStatusHMC",
                               plot_group_order = c("low-risk", "high-risk"),
                               plot_group_ellipse = TRUE) + theme_bw() +
  theme(panel.grid.major = element_blank(), panel.grid.minor = element_blank(),
        legend.title = element_blank(), text = element_text(size = 16))
t_capt_beta_bc$cal_ordination(ordination = "PCoA")
t_capt_beta_bc$plot_ordination(color_values = capt_colors,
                               plot_color = "ConservationStatusHMC", 
                               plot_shape = "ConservationStatusHMC",
                               plot_group_order = c("low-risk", "high-risk"),
                               plot_group_ellipse = TRUE) + theme_bw() +
  theme(panel.grid.major = element_blank(), panel.grid.minor = element_blank(),
        legend.title = element_blank(), text = element_text(size = 16))
t_capt_beta_uu$cal_ordination(ordination = "PCoA")
t_capt_beta_uu$plot_ordination(color_values = capt_colors,
                               plot_color = "ConservationStatusHMC", 
                               plot_shape = "ConservationStatusHMC",
                               plot_group_order = c("low-risk", "high-risk"),
                               plot_group_ellipse = TRUE) + theme_bw() +
  theme(panel.grid.major = element_blank(), panel.grid.minor = element_blank(),
        legend.title = element_blank(), text = element_text(size = 16))
t_capt_beta_wu$cal_ordination(ordination = "PCoA")
t_capt_beta_wu$plot_ordination(color_values = capt_colors,
                               plot_color = "ConservationStatusHMC", 
                               plot_shape = "ConservationStatusHMC",
                               plot_group_order = c("low-risk", "high-risk"),
                               plot_group_ellipse = TRUE) + theme_bw() +
  theme(panel.grid.major = element_blank(), panel.grid.minor = element_blank(),
        legend.title = element_blank(), text = element_text(size = 16))

#PCoA plots for wild animals
t_wild_beta_jc$cal_ordination(ordination = "PCoA")
t_wild_beta_jc$plot_ordination(color_values = wild_colors,
                               plot_color = "ConservationStatusHMC", 
                               plot_shape = "ConservationStatusHMC",
                               plot_group_order = c("low-risk", "high-risk"),
                               plot_group_ellipse = TRUE) + theme_bw() +
  theme(panel.grid.major = element_blank(), panel.grid.minor = element_blank(),
        legend.title = element_blank(), text = element_text(size = 16))
t_wild_beta_bc$cal_ordination(ordination = "PCoA")
t_wild_beta_bc$plot_ordination(color_values = wild_colors,
                               plot_color = "ConservationStatusHMC", 
                               plot_shape = "ConservationStatusHMC",
                               plot_group_order = c("low-risk", "high-risk"),
                               plot_group_ellipse = TRUE) + theme_bw() +
  theme(panel.grid.major = element_blank(), panel.grid.minor = element_blank(),
        legend.title = element_blank(), text = element_text(size = 16))
t_wild_beta_uu$cal_ordination(ordination = "PCoA")
t_wild_beta_uu$plot_ordination(color_values = wild_colors,
                               plot_color = "ConservationStatusHMC", 
                               plot_shape = "ConservationStatusHMC",
                               plot_group_order = c("low-risk", "high-risk"),
                               plot_group_ellipse = TRUE) + theme_bw() +
  theme(panel.grid.major = element_blank(), panel.grid.minor = element_blank(),
        legend.title = element_blank(), text = element_text(size = 16))
t_wild_beta_wu$cal_ordination(ordination = "PCoA")
t_wild_beta_wu$plot_ordination(color_values = wild_colors,
                               plot_color = "ConservationStatusHMC", 
                               plot_shape = "ConservationStatusHMC",
                               plot_group_order = c("low-risk", "high-risk"),
                               plot_group_ellipse = TRUE) + theme_bw() +
  theme(panel.grid.major = element_blank(), panel.grid.minor = element_blank(),
        legend.title = element_blank(), text = element_text(size = 16))

#Beta diversity distance barplots for captive mammals
t_capt_beta_jc$cal_group_distance()
t_capt_beta_jc$plot_group_distance(color_values = capt_colors,
                                   plot_group_order = c("low-risk", "high-risk"),
                                   pair_compare_method = "anova") + ylim(0,1)
t_capt_beta_bc$cal_group_distance()
t_capt_beta_bc$plot_group_distance(color_values = capt_colors,
                                   plot_group_order = c("low-risk", "high-risk"),
                                   pair_compare_method = "anova") + ylim(0,1)
t_capt_beta_uu$cal_group_distance()
t_capt_beta_uu$plot_group_distance(color_values = capt_colors,
                                   plot_group_order = c("low-risk", "high-risk"),
                                   pair_compare_method = "anova") + ylim(0,1)
t_capt_beta_wu$cal_group_distance()
t_capt_beta_wu$plot_group_distance(color_values = capt_colors,
                                   plot_group_order = c("low-risk", "high-risk"),
                                   pair_compare_method = "anova") + ylim(0,1)

#Beta diversity distance barplots for wild mammals
t_wild_beta_jc$cal_group_distance()
t_wild_beta_jc$plot_group_distance(color_values = wild_colors,
                                   plot_group_order = c("low-risk", "high-risk"),
                                   pair_compare_method = "anova") + ylim(0,1)
t_wild_beta_bc$cal_group_distance()
t_wild_beta_bc$plot_group_distance(color_values = wild_colors,
                                   plot_group_order = c("low-risk", "high-risk"),
                                   pair_compare_method = "anova") + ylim(0,1)
t_wild_beta_uu$cal_group_distance()
t_wild_beta_uu$plot_group_distance(color_values = wild_colors,
                                   plot_group_order = c("low-risk", "high-risk"),
                                   pair_compare_method = "anova") + ylim(0,1)
t_wild_beta_wu$cal_group_distance()
t_wild_beta_wu$plot_group_distance(color_values = wild_colors,
                                   plot_group_order = c("low-risk", "high-risk"),
                                   pair_compare_method = "anova") + ylim(0,1)


## Beta diversity plots according to conservation status:
#Bray-Curtis PCoA plot based on conservation
t_cons_beta_bc$cal_ordination(ordination = "PCoA")
t_cons_beta_bc$plot_ordination(color_values = RColorBrewer::brewer.pal(8, "Set1"),
                               plot_color = "ConservationStatusHMC", 
                               plot_shape = "ConservationStatusHMC",
                               plot_group_order = c("LeastConcern",
                                                    "NearThreatened",
                                                    "Vulnerable",
                                                    "Endangered",
                                                    "CriticallyEndangered"),
                               plot_group_ellipse = TRUE) + theme_bw() +
  theme(panel.grid.major = element_blank(), panel.grid.minor = element_blank())